@@ -1,7 +1,7 @@
 /*
  * Copyright 2005-2010 Gentoo Foundation
  * Distributed under the terms of the GNU General Public License v2
- * $Header: /var/cvsroot/gentoo-projects/portage-utils/qmerge.c,v 1.116 2012/08/13 22:23:35 robbat2 Exp $
+ * $Header: /var/cvsroot/gentoo-projects/portage-utils/qmerge.c,v 1.128 2013/04/30 01:50:20 vapier Exp $
  *
  * Copyright 2005-2010 Ned Ludd        - <solar@gentoo.org>
  * Copyright 2005-2010 Mike Frysinger  - <vapier@gentoo.org>
@@ -65,7 +65,7 @@ static const char * const qmerge_opts_help[] = {
 	COMMON_OPTS_HELP
 };
 
-static const char qmerge_rcsid[] = "$Id: qmerge.c,v 1.116 2012/08/13 22:23:35 robbat2 Exp $";
+static const char qmerge_rcsid[] = "$Id: qmerge.c,v 1.128 2013/04/30 01:50:20 vapier Exp $";
 #define qmerge_usage(ret) usage(ret, QMERGE_FLAGS, qmerge_long_opts, qmerge_opts_help, lookup_applet_idx("qmerge"))
 
 char search_pkgs = 0;
@@ -182,21 +182,23 @@ _q_static void qmerge_initialize(void)
 	if (!search_pkgs && !pretend) {
 		if (mkdir_p(pkgdir, 0755))
 			errp("could not setup PKGDIR: %s", pkgdir);
-		mkdir_p(port_tmpdir, 0755);
 	}
 
-	xchdir(port_tmpdir);
-	xchdir("portage");
+	char *buf;
+	xasprintf(&buf, "%s/portage", port_tmpdir);
+	mkdir_p(buf, 0755);
+	xchdir(buf);
 
-	if (force_download && force_download != 2)
-		unlink(Packages);
+	if (force_download != 2) {
+		if (force_download)
+			unlink(Packages);
 
-	if ((access(Packages, R_OK) != 0) && (force_download != 2)) {
-		char *tbuf = NULL;
-		xasprintf(&tbuf, "%s/portage/", port_tmpdir);
-		if (access(Packages, R_OK) != 0)
-			fetch(tbuf, Packages);
-		free(tbuf);
+		if (access(Packages, R_OK) != 0) {
+			xasprintf(&buf, "%s/portage/", port_tmpdir);
+			if (access(Packages, R_OK) != 0)
+				fetch(buf, Packages);
+			free(buf);
+		}
 	}
 }
 
@@ -210,19 +212,21 @@ struct qmerge_bv_state {
 _q_static int qmerge_filter_cat(q_vdb_cat_ctx *cat_ctx, void *priv)
 {
 	struct qmerge_bv_state *state = priv;
-	return strcmp(cat_ctx->name, state->catname) == 0;
+	return !state->catname || strcmp(cat_ctx->name, state->catname) == 0;
 }
 
 _q_static int qmerge_best_version_cb(q_vdb_pkg_ctx *pkg_ctx, void *priv)
 {
 	struct qmerge_bv_state *state = priv;
-	if (qlist_match(pkg_ctx, state->buf, true))
-		strcpy(state->retbuf, state->buf);
+	if (qlist_match(pkg_ctx, state->buf, NULL, true))
+		snprintf(state->retbuf, sizeof(state->buf), "%s/%s",
+			pkg_ctx->cat_ctx->name, pkg_ctx->name);
 	return 0;
 }
 
 _q_static char *best_version(const char *catname, const char *pkgname)
 {
+	static int vdb_check = 1;
 	static char retbuf[4096];
 	struct qmerge_bv_state state = {
 		.catname = catname,
@@ -230,10 +234,28 @@ _q_static char *best_version(const char *catname, const char *pkgname)
 		.retbuf = retbuf,
 	};
 
-	snprintf(state.buf, sizeof(state.buf), "%s/%s", catname, pkgname);
+	/* Make sure these dirs exist before we try walking them */
+	switch (vdb_check) {
+	case 1: {
+		int fd = open(portroot, O_RDONLY|O_CLOEXEC);
+		if (fd >= 0) {
+			/* skip leading slash */
+			vdb_check = faccessat(fd, portvdb + 1, X_OK, 0);
+			close(fd);
+		} else
+			vdb_check = -1;
+	}
+		if (vdb_check)
+	case -1:
+			goto done;
+	}
+
 	retbuf[0] = '\0';
+	snprintf(state.buf, sizeof(state.buf), "%s%s%s",
+		catname ? : "", catname ? "/" : "", pkgname);
 	q_vdb_foreach_pkg(qmerge_best_version_cb, &state, qmerge_filter_cat);
 
+ done:
 	return retbuf;
 }
 
@@ -385,7 +407,8 @@ qprint_tree_node(int level, const depend_atom *atom, const struct pkg_t *pkg)
 	return c;
 }
 
-_q_static void pkg_run_func(const char *vdb_path, const char *phases, const char *func)
+_q_static void
+pkg_run_func(const char *vdb_path, const char *phases, const char *func, const char *D, const char *T)
 {
 	const char *phase;
 	char *script;
@@ -409,24 +432,37 @@ _q_static void pkg_run_func(const char *vdb_path, const char *phases, const char
 		"debug-print-section() { :; }\n"
 		/* Not quite right */
 		"has_version() { qlist -ICq -e '$1' >/dev/null; }\n"
+		/* best_version() */
 		"use() { useq \"$@\"; }\n"
-		"useq() { hasq $1 $USE; }\n"
+		"usex() { useq \"$1\" && echo \"${2-yes}$4\" || echo \"${3-no}$5\"; }\n"
+		"useq() { hasq \"$1\" $USE; }\n"
 		"has() { hasq \"$@\"; }\n"
 		"hasq() { local h=$1; shift; case \" $* \" in *\" $h \"*) return 0;; *) return 1;; esac; }\n"
+		"hasv() { hasq \"$@\" && echo \"$1\"; }\n"
 		"elog() { printf ' * %%b\\n' \"$*\"; }\n"
 		"einfo() { elog \"$@\"; }\n"
 		"ewarn() { elog \"$@\"; }\n"
+		"eqawarn() { elog \"QA: \"\"$@\"; }\n"
 		"eerror() { elog \"$@\"; }\n"
 		"die() { eerror \"$@\"; exit 1; }\n"
 		"ebegin() { printf ' * %%b ...' \"$*\"; }\n"
-		"eend() { local r=${1:-$?}; [ $r -eq 0 ] && echo '[ ok ]' || echo '[ !! ]'; return $r; }\n"
+		"eend() { local r=${1:-$?}; [ $# -gt 0 ] && shift; [ $r -eq 0 ] && echo ' [ ok ]' || echo \" $* \"'[ !! ]'; return $r; }\n"
 		/* Unpack the env if need be */
 		"[ -e '%1$s/environment' ] || { bzip2 -dc '%1$s/environment.bz2' > '%1$s/environment' || exit 1; }\n"
-		/* Load the env and run the func */
-		". '%1$s/environment' && %2$s\n"
+		/* Load the main env */
+		". '%1$s/environment'\n"
+		/* Reload env vars that matter to us */
+		"MERGE_TYPE=binary\n"
+		"ROOT='%4$s'\n"
+		"EROOT=\"${EPREFIX%%/}/${ROOT#/}\"\n"
+		"D='%5$s'\n"
+		"ED=\"${EPREFIX%%/}/${D#/}\"\n"
+		"T='%6$s'\n"
+		/* Finally run the func */
+		"%2$s\n"
 		/* Ignore func return values (not exit values) */
 		":",
-		vdb_path, func, phase);
+		vdb_path, func, phase, portroot, D, T);
 	xsystembash(script);
 	free(script);
 }
@@ -531,7 +567,7 @@ merge_tree_at(int fd_src, const char *src, int fd_dst, const char *dst,
 
 			/* syntax: obj filename hash mtime */
 			hash = hash_file_at(subfd_src, name, HASH_MD5);
-			fprintf(contents, "obj %s %s %lu\n", cpath, hash, (unsigned long)st.st_mtime);
+			fprintf(contents, "obj %s %s %"PRIu64"u\n", cpath, hash, (uint64_t)st.st_mtime);
 
 			/* Check CONFIG_PROTECT */
 			if (config_protected(cpath, cp_argc, cp_argv, cpm_argc, cpm_argv)) {
@@ -629,7 +665,7 @@ merge_tree_at(int fd_src, const char *src, int fd_dst, const char *dst,
 			sym[len] = '\0';
 
 			/* syntax: sym src -> dst mtime */
-			fprintf(contents, "sym %s -> %s %lu\n", cpath, sym, (unsigned long)st.st_mtime);
+			fprintf(contents, "sym %s -> %s %"PRIu64"u\n", cpath, sym, (uint64_t)st.st_mtime);
 			qprintf("%s>>>%s %s%s -> %s%s\n", GREEN, NORM, CYAN, cpath, sym, NORM);
 			*objs = add_set(cpath, "", *objs);
 
@@ -701,7 +737,7 @@ pkg_merge(int level, const depend_atom *atom, const struct pkg_t *pkg)
 	queue *objs;
 	FILE *fp, *contents;
 	char buf[1024], phases[128];
-	char *tbz2, *p;
+	char *tbz2, *p, *D, *T;
 	int i;
 	char **ARGV;
 	int ARGC;
@@ -818,14 +854,19 @@ pkg_merge(int level, const depend_atom *atom, const struct pkg_t *pkg)
 		return;
 
 	/* Set up our temp dir to unpack this stuff */
-	xasprintf(&p, "%s/qmerge/%s", port_tmpdir, pkg->PF);
+	xasprintf(&p, "%s/qmerge/%s/%s", port_tmpdir, pkg->CATEGORY, pkg->PF);
 	mkdir_p(p, 0755);
 	xchdir(p);
+	xasprintf(&D, "%s/image", p);
+	xasprintf(&T, "%s/temp", p);
 	free(p);
 
 	/* Doesn't actually remove $PWD, just everything under it */
 	rm_rf(".");
 
+	mkdir("temp", 0755);
+	mkdir_p(portroot, 0755);
+
 	/* XXX: maybe some day we should have this step operate on the
 	 *      tarball directly rather than unpacking it first. */
 
@@ -846,7 +887,9 @@ pkg_merge(int level, const depend_atom *atom, const struct pkg_t *pkg)
 	fflush(stdout);
 
 	eat_file("vdb/DEFINED_PHASES", phases, sizeof(phases));
-	pkg_run_func("vdb", phases, "pkg_preinst");
+	pkg_run_func("vdb", phases, "pkg_pretend", D, T);
+	pkg_run_func("vdb", phases, "pkg_setup", D, T);
+	pkg_run_func("vdb", phases, "pkg_preinst", D, T);
 
 	/* XXX: kill this off */
 	xchdir("image");
@@ -873,13 +916,14 @@ pkg_merge(int level, const depend_atom *atom, const struct pkg_t *pkg)
 	if ((contents = fopen("vdb/CONTENTS", "w")) == NULL)
 		errf("come on wtf?");
 	objs = NULL;
-	merge_tree("image", portroot, contents, &objs, iargc, iargv);
+	if (merge_tree("image", portroot, contents, &objs, iargc, iargv))
+		errp("failed to merge to %s", portroot);
 	fclose(contents);
 
 	freeargv(iargc, iargv);
 
 	/* run postinst */
-	pkg_run_func("vdb", phases, "pkg_postinst");
+	pkg_run_func("vdb", phases, "pkg_postinst", D, T);
 
 	/* XXX: hmm, maybe we'll want to strip more ? */
 	unlink("vdb/environment");
@@ -938,6 +982,8 @@ pkg_merge(int level, const depend_atom *atom, const struct pkg_t *pkg)
 		free(q->item);
 		free(q);
 	}
+	free(D);
+	free(T);
 
 	/* Update the magic counter */
 	if ((fp = fopen("vdb/COUNTER", "w")) != NULL) {
@@ -968,7 +1014,7 @@ _q_static int
 pkg_unmerge(const char *cat, const char *pkgname, queue *keep)
 {
 	size_t buflen;
-	char *buf, *vdb_path, phases[128];
+	char *buf, *vdb_path, *T, phases[128];
 	FILE *fp;
 	int ret, fd, vdb_fd, portroot_fd;
 	int cp_argc, cpm_argc;
@@ -1001,6 +1047,7 @@ pkg_unmerge(const char *cat, const char *pkgname, queue *keep)
 	/* Get a handle on the vdb path which we'll use everywhere else */
 	/* Note: This vdb_path must be absolute since we use it in pkg_run_func() */
 	xasprintf(&vdb_path, "%s%s/%s/%s/", portroot, portvdb, cat, pkgname);
+	xasprintf(&T, "%stemp", vdb_path);
 	vdb_fd = openat(portroot_fd, vdb_path, O_RDONLY | O_CLOEXEC);
 	if (vdb_fd == -1) {
 		warnp("unable to read %s", vdb_path);
@@ -1010,7 +1057,8 @@ pkg_unmerge(const char *cat, const char *pkgname, queue *keep)
 	/* First execute the pkg_prerm step */
 	if (!pretend) {
 		eat_file_at(vdb_fd, "DEFINED_PHASES", phases, sizeof(phases));
-		pkg_run_func(vdb_path, phases, "pkg_prerm");
+		mkdir_p(T, 0755);
+		pkg_run_func(vdb_path, phases, "pkg_prerm", T, T);
 	}
 
 	/* Now start removing all the installed files */
@@ -1030,6 +1078,8 @@ pkg_unmerge(const char *cat, const char *pkgname, queue *keep)
 	makeargv(config_protect, &cp_argc, &cp_argv);
 	makeargv(config_protect_mask, &cpm_argc, &cpm_argv);
 
+	bool unmerge_config_protected = !!strstr(features, "config-protect-if-modified");
+
 	while (getline(&buf, &buflen, fp) != -1) {
 		queue *q;
 		contents_entry *e;
@@ -1049,17 +1099,21 @@ pkg_unmerge(const char *cat, const char *pkgname, queue *keep)
 
 		/* Should we remove in order symlinks,objects,dirs ? */
 		switch (e->type) {
-			case CONTENTS_DIR:
-				if (!protected) {
-					/* since the dir contains files, we remove it later */
-					llist_char *list = xmalloc(sizeof(llist_char));
-					list->data = xstrdup(e->name);
-					list->next = dirs;
-					dirs = list;
-				}
+			case CONTENTS_DIR: {
+				/* since the dir contains files, we remove it later */
+				llist_char *list = xmalloc(sizeof(llist_char));
+				list->data = xstrdup(e->name);
+				list->next = dirs;
+				dirs = list;
 				continue;
+			}
 
 			case CONTENTS_OBJ:
+				if (protected && unmerge_config_protected) {
+					/* If the file wasn't modified, unmerge it */
+					unsigned char *hash = hash_file_at(portroot_fd, e->name + 1, HASH_MD5);
+					protected = strcmp(e->digest, (const char *)hash);
+				}
 				break;
 
 			case CONTENTS_SYM:
@@ -1135,7 +1189,8 @@ pkg_unmerge(const char *cat, const char *pkgname, queue *keep)
 
 	if (!pretend) {
 		/* Then execute the pkg_postrm step */
-		pkg_run_func(vdb_path, phases, "pkg_postrm");
+		pkg_run_func(vdb_path, phases, "pkg_postrm", T, T);
+		rm_rf(T);
 
 		/* Finally delete the vdb entry */
 		rm_rf_at(portroot_fd, vdb_path);
@@ -1153,6 +1208,7 @@ pkg_unmerge(const char *cat, const char *pkgname, queue *keep)
 	if (portroot_fd != -1)
 		close(portroot_fd);
 	free(buf);
+	free(T);
 	free(vdb_path);
 
 	return ret;
@@ -1446,6 +1502,26 @@ unmerge_packages(queue *todo)
 	return 0;
 }
 
+_q_static FILE *
+open_binpkg_index(void)
+{
+	FILE *fp;
+	char buf[BUFSIZ];
+
+	snprintf(buf, sizeof(buf), "%s/portage/%s", port_tmpdir, Packages);
+	fp = fopen(buf, "r");
+	if (fp)
+		return fp;
+
+	snprintf(buf, sizeof(buf), "%s/%s", pkgdir, Packages);
+	fp = fopen(buf, "r");
+	if (fp)
+		return fp;
+
+	errp("Unable to open package file %s in %s/portage or %s",
+		Packages, port_tmpdir, pkgdir);
+}
+
 _q_static struct pkg_t *
 grab_binpkg_info(const char *name)
 {
@@ -1454,19 +1530,14 @@ grab_binpkg_info(const char *name)
 	char *p;
 	depend_atom *atom;
 
-	struct pkg_t *pkg = xmalloc(sizeof(struct pkg_t));
-	struct pkg_t *rpkg = xmalloc(sizeof(struct pkg_t));
+	struct pkg_t *pkg = xzalloc(sizeof(struct pkg_t));
+	struct pkg_t *rpkg = xzalloc(sizeof(struct pkg_t));
 
 	static char best_match[sizeof(Pkg.PF)+2+sizeof(Pkg.CATEGORY)];
 
 	best_match[0] = 0;
 
-	memset(pkg, 0, sizeof(struct pkg_t));
-	memset(rpkg, 0, sizeof(struct pkg_t));
-	snprintf(buf, sizeof(buf), "%s/portage/Packages", port_tmpdir);
-
-	if ((fp = fopen(buf, "r")) == NULL)
-		errp("Unable to open package file %s", buf);
+	fp = open_binpkg_index();
 
 	while (fgets(buf, sizeof(buf), fp) != NULL) {
 		if (*buf == '\n') {
@@ -1567,10 +1638,7 @@ find_binpkg(const char *name)
 	if (NULL == name)
 		return best_match;
 
-	snprintf(buf, sizeof(buf), "%s/portage/Packages", port_tmpdir);
-
-	if ((fp = fopen(buf, "r")) == NULL)
-		errp("Unable to open package file %s", buf);
+	fp = open_binpkg_index();
 
 	while (fgets(buf, sizeof(buf), fp) != NULL) {
 		if (*buf == '\n') {
@@ -1643,8 +1711,7 @@ parse_packages(queue *todo)
 	char *buf, *p;
 	long lineno = 0;
 
-	if ((fp = fopen(Packages, "r")) == NULL)
-		errp("Unable to open package file %s", Packages);
+	fp = open_binpkg_index();
 
 	memset(&Pkg, 0, sizeof(Pkg));
 
